#!/usr/bin/perl -w

############################################################################
#    Copyright (C) 2005 by Frank Seidel   #
#    frank@f-seidel.de   #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################


#
use strict;
use warnings;
use Getopt::Long;


#program version
my $LICENCE="GPL";
my $VERSION="0.1";
my $AUTHOR="Frank Seidel <frank\@f-seidel.de>";

#menutestoutput-file
my $menutestout="menutestout";

#icon-searchpathes
my $iconpathes="/usr/share/pixmaps /opt/kde3/share /opt/gnome/share/pixmaps";

#chaches
my %menupathcache;
my %menunamecache;
my %desktopcache;

#just saves the output from kbuildsycoca
sub call_menutest {
	my $result=0;
	system("kbuildsycoca --menutest >$menutestout 2>/dev/null");

	if (-r "$menutestout") {$result=1;}

	return $result;
}

#returns array with all application-id's
sub get_appids {
	my @result;
	my $line;

	#check if menutest is there
	if (-r "$menutestout" or call_menutest()) {
		open(MTO,$menutestout) or die("could not open $menutestout : $!\n");
			while (defined($line=<MTO>)) {
			 chomp($line);
			 $line =~ s/^[^\t]+\t([^\t]+)\t.+$/$1/;
			 push(@result,$line);
			}
		close(MTO) or die("could not close $menutestout : $!\n");
	}
	
	if (@result > 0) {@result=sort(@result);}
	
	return @result;
}

#returns the application-id of the given .desktop-file
sub desktopToAppid {
  my ($desktopfile)=@_;
  my $line;
  my $found=0;
  my $result="";
  
  if (-r "$menutestout" or call_menutest()) {
    open(MTO,$menutestout) or die("could not open $menutestout : $!\n");
    while (defined($line=<MTO>) and $found == 0) {
      chomp($line);
      if ($line =~ /$desktopfile/i) {
        $line =~ s/^[^\t]+\t([^\t]+)\t.+$/$1/;
        $result=$line;$found=1;
      }
    }
    close(MTO) or die("could not close $menutestout : $!\n");
  }
  
  return $result;
}

#return the absolute path of the .desktop-file for the given application-id
sub getDesktopfile {
  my ($appid)=@_;
  my $result="";
  my $found=0;
  my $line;
  
  if (! defined($desktopcache{$appid})) {
    if (-r "$menutestout" or call_menutest()) {
      open(MTO,$menutestout) or die("could not open $menutestout : $!\n");
        while (defined($line=<MTO>) and $found == 0) {
          chomp($line);
          if ($line =~ /^[^\t]+\t$appid/) {
            $line =~ s/^[^\t]+\t[^\t]+\t(.+)$/$1/;
            $result=$line;
            $found=1;
            $desktopcache{$appid}=$result;
          }
        }
      close(MTO) or die("could not close $menutestout : $!\n");
    }
  } else {
    $result=$desktopcache{$appid};
  }
  
  return $result;
}

#return the menupath of the given application-id
sub getMenupath {
  my ($appid)=@_;
  my $result="";
  
  if (! defined($menupathcache{$appid})) {
    $result=`kde-menu --nocache-update --print-menu-name $appid 2>/dev/null`;
    chomp($result);
    $menupathcache{$appid}=$result;
  } else {
    $result=$menupathcache{$appid};
  }
  
  return $result;
}

#return the menus name of the given application-id
sub getMenuname {
  my ($appid)=@_;
  my $result="";
  my $desktopfile;
  my $lang;
  my $line;
  my $found=0;
  
  if (! defined($menunamecache{$appid})) {
    $desktopfile=getDesktopfile($appid);
    $lang=$ENV{"LANG"};$lang =~ s/^(..).*$/$1/;$lang =~ tr/A-Z/a-z/;
    
    open(DTF,$desktopfile) or die("could not open file $desktopfile : $!\n");
      while (defined($line=<DTF>) and $found == 0) {
        if ($line =~ /^Name\=/) {
          chomp($line);
          $line =~ s/^Name\=(.+)$/$1/;
          $result=$line;
        } elsif ($line =~ /^Name\[${lang}\]\=/) {
          chomp($line);
          $line =~ s/^Name\[..\]\=(.+)$/$1/;
          $result=$line;
          $found=1;
        } elsif ($line =~ /^\[.+\]/ and $line !~ /^\[(KDE )?Desktop Entry\]/) {
          $found=1;
        }
      }
    close(DTF) or die("could not close file $desktopfile : $!\n");
    $menunamecache{$appid}=$result;
  } else {
    $result=$menunamecache{$appid};
  }
  
  return $result;
}

#returns true if the given application-id is currently shown
sub isShown {
  my ($appid)=@_;
  my $result=1;
  
  if (getMenupath($appid) eq "") {$result=0;}
  
  return $result;
}

#returns the .desktop-files Exec-entry of the given appid
sub getExec {
  my ($appid)=@_;
  my $desktopfile;
  my $line;
  my $indesktop=0;
  my $found=0;
  my $result="";
  
  $desktopfile = getDesktopfile($appid);
  if ($desktopfile ne "" and -r "$desktopfile") {
    open(DTF,$desktopfile) or die("could not open file $desktopfile : $!\n");
    while (defined($line=<DTF>) and $found == 0) {
      chomp($line);
      #check for right section
      if ($line =~ /^\[(KDE )?Desktop Entry\]/) {
        $indesktop=1;
      } elsif ($line =~ /^\[.+\]/) {
        if ($indesktop) {$indesktop=0;}
      } elsif ($line =~ /^Exec\=/) {
        if ($indesktop) {
          $result=$line;
          $result =~ s/^Exec\=(.+)$/$1/;
          $found=1;
        }
      }
    }
    close(DTF) or die("could not close file $desktopfile : $!\n");
  } else {
    print STDERR "WARNING: could not access desktopfile for app-id $appid\n";
  }
  
  return $result;
}

#returns the .desktop-files Icon-entry of the given appid
sub getIcon {
  my ($appid)=@_;
  my $desktopfile;
  my $line;
  my $indesktop=0;
  my $found=0;
  my $result="";
  
  $desktopfile = getDesktopfile($appid);
  if ($desktopfile ne "" and -r "$desktopfile") {
    open(DTF,$desktopfile) or die("could not open file $desktopfile : $!\n");
    while (defined($line=<DTF>) and $found == 0) {
      chomp($line);
      #check for right section
      if ($line =~ /^\[(KDE )?Desktop Entry\]/) {
        $indesktop=1;
      } elsif ($line =~ /^\[.+\]/) {
        if ($indesktop) {$indesktop=0;}
      } elsif ($line =~ /^Icon\=/) {
        if ($indesktop) {
          $result=$line;
          $result =~ s/^Icon\=(.+)$/$1/;
          $found=1;
        }
      }
    }
    close(DTF) or die("could not close file $desktopfile : $!\n");
  } else {
    print STDERR "WARNING: could not access desktopfile for app-id $appid\n";
  }
  
  return $result;
}


#returns true if the given icon-name could be found
sub checkIcon {
  my ($icon)=@_;
  my $result=0;
  
  if (defined($icon) and $icon ne "") {
    if ($icon !~ /^\//) {
      if (system("find $iconpathes -path \"*$icon*\" | grep $icon &>/dev/null") == 0) {$result=1;}
    } else {
      if (-r "$icon") {$result=1;}
    }
  }
  
  return $result;
}

#returns true if the given exec-name could be found
sub checkExec {
  my ($exec)=@_;
  my $result=0;
  
  if (defined($exec) and $exec ne "") {
    $exec =~ s/^(\S+)\s*.*$/$1/;
    if (system("which $exec &>/dev/null") == 0) {$result=1;}
  }
  
  return $result;
}

#returns true if a given desktop-file (absolute path) passes some general tests
sub generalDesktopfileCheck {
  my ($dfile)=@_;
  my $appid=desktopToAppid($dfile);
  my $result=0;
  my $error="";
  my $line;
  
  if (-r "$dfile") {
    open(DTF,$dfile) or die("could not open $dfile : $!\n");
    while (defined($line=<DTF>) and $error eq "") {
      chomp($line);
      if ($line =~ /^\[/) {
          if ($line !~ /^\[[^\]]+\]/) {$error="corrupted section-line:$line";}
      } elsif ($line =~ /^[a-zA-Z]+/) {
            if ($line !~ /^[a-zA-Z]+.*\=/) {$error="missing equal-char in line:$line";}
            elsif ($line =~ /\"/) { if ($line !~ /^.*[^\\\"]\".*[^\\]\".*$/) {$error="wrong quotes in line:$line";}    }
      } 
    }
    close(DTF) or die("could not close $dfile : $!\n");
  }
  
  if ($error eq "") {
    $result=1;
  } else {
    print STDERR "WARNING:error in (" . getMenupath($appid) . "/" . getMenuname($appid) . ") ${dfile}:$error\n";
  }
  
  return $result;
}

#prints the shown kde-menustructure
sub printMenustructure {
  my $appid;
  my $path;
  my $name;
  my @output;
  foreach $appid (get_appids()) {
    if (isShown($appid)) {
      $path=getMenupath($appid);
      $name=getMenuname($appid);
      push(@output,"$path/$name");
    }
  }
  @output=sort(@output);
  foreach $name (@output) {
    print("$name\n");
  }
}

#prints available application-ids
sub printAppids {
  my $appid;
  foreach $appid (get_appids()) {
    print("$appid\n");
  }
}

#prints the available .desktop-files
sub printDesktopfiles {
  my $appid;
  my $line;
  my @output;
  foreach $appid (get_appids()) {
    push(@output, getDesktopfile($appid));
  }
  @output=sort(@output);
  foreach $line (@output) {
    print("$line\n");
  }
}

#prints the exec-entries of the shown menu-entries
sub printExecs {
  my $appid;
  my @output;
  my $exec;
  foreach $appid (get_appids()) {
    if (isShown($appid)) {
      push(@output,getExec($appid));
    }
  }
  @output=sort(@output);
  foreach $exec (@output) {
    print("$exec\n");
  }
}

#does a simple check of the visible (in kde-menu) .desktop-files
sub simpleCheck {
  my @appids=get_appids();
  my $appid;
  my $desktopfile;
  my $allok=1;
  
  foreach $appid (@appids) {
    if (isShown($appid)) {
      $desktopfile=getDesktopfile($appid);
      if (! generalDesktopfileCheck($desktopfile)) {$allok=0;}
    }
  }
  
  return $allok;
}

#does a more extended check of the visible .desktop-files
sub extendedCheck {
  my $result=1;
  my @apps=get_appids();
  my $desktopfile;
  my $appid;
  my $icon;
  my $exec;
  
  foreach $appid (@apps) {
    if (isShown($appid)) {
      #first we do the simple check
      $desktopfile=getDesktopfile($appid);
      if (! generalDesktopfileCheck($desktopfile)) {$result=0;}
      $icon=getIcon($appid);
      if ($icon eq "") {
        print("WARNING:missing icon-definition for " . getMenupath($appid) . "/" . getMenuname($appid) . " (app-id:${appid})\n");
        $result=0;
      } else {
        if (! checkIcon($icon)) {
          print("WARNING:icon-file missing for " . getMenupath($appid) . "/" . getMenuname($appid) . " (app-id:${appid}/icon:${icon})\n");
          $result=0;
        }
      }
      $exec=getExec($appid);
      if ($exec eq "") {
        print("WARNING:missing Exec-definition for " . getMenupath($appid) . "/" . getMenuname($appid) . " (app-id:${appid})\n");
        $result=0;
      } else {
        if (! checkExec($exec)) {
          print("WARNING:Executable-file missing for " . getMenupath($appid) . "/" . getMenuname($appid) . " (app-id:${appid}/Exec:${exec})\n");
          $result=0;
        }
      }
    }
  }
  
  return $result;
}

#does a check of all .desktop-files
sub completeCheck {
  my $result=1;
  my @apps=get_appids();
  my $desktopfile;
  my $appid;
  my $icon;
  my $exec;
  
  foreach $appid (@apps) {
      #first we do the simple check
      $desktopfile=getDesktopfile($appid);
      if (! generalDesktopfileCheck($desktopfile)) {$result=0;}
      $icon=getIcon($appid);
      if ($icon eq "") {
        print("WARNING:missing icon-definition for " . getMenupath($appid) . "/" . getMenuname($appid) . " (app-id:${appid})\n");
        $result=0;
      } else {
        if (! checkIcon($icon)) {
          print("WARNING:icon-file missing for " . getMenupath($appid) . "/" . getMenuname($appid) . " (app-id:${appid}/icon:${icon})\n");
          $result=0;
        }
      }
      $exec=getExec($appid);
      if ($exec eq "") {
        print("WARNING:missing Exec-definition for " . getMenupath($appid) . "/" . getMenuname($appid) . " (app-id:${appid})\n");
        $result=0;
      } else {
        if (! checkExec($exec)) {
          print("WARNING:Executable-file missing for " . getMenupath($appid) . "/" . getMenuname($appid) . " (app-id:${appid}/Exec:${exec})\n");
          $result=0;
        }
      }
  }
  
  return $result;
}

#walk through (highlight) all visible kde-menu entries
sub walkThrough {
  my $appid;
  my $path;
  my $name;
  my @output;
  foreach $appid (get_appids()) {
    if (isShown($appid)) {
      $path=getMenupath($appid);
      $name=getMenuname($appid);
      push(@output,"$path/$name:$appid");
    }
  }
  @output=sort(@output);
  foreach $appid (@output) {
    $path=$appid; $path =~ s/^([^\:]+)\:.+$/$1/;
    $appid =~ s/^[^\:]+\:(.+)$/$1/;
    print("selecting $path\n");
    system("kde-menu --nocache-update --highlight $appid");
  }
}

#MAIN-Part of Application=>
my $printmenu=0;
my $printdesktopfiles=0;
my $printhelp=0;
my $simpledesktopcheck=0;
my $extendedcheck=0;
my $completecheck=0;
my $walkthrough=0;

if (GetOptions('printmenu' => \$printmenu,
                'desktopfiles' => \$printdesktopfiles,
                'simplecheck' => \$simpledesktopcheck,
                'extendedcheck' => \$extendedcheck,
                'completecheck' => \$completecheck,
                'walkthrough' => \$walkthrough,
                'help' => \$printhelp)) {
  if ($printmenu) {
    printMenustructure();
    exit 0;
  } elsif ($walkthrough) {
    walkThrough();
    exit 0;
  } elsif ($printdesktopfiles) {
    printDesktopfiles();
    exit 0;
  } elsif ($extendedcheck) {
    if (extendedCheck()) {
      print("PASSED\n");
      exit 0;
    } else {
      print(">>FAILED<<\n");
      exit 1;
    }
  } elsif ($completecheck) {
    if (completeCheck()) {
      print("PASSED\n");
      exit 0;
    } else {
      print(">>FAILED<<\n");
      exit 1;
    }
  } elsif ($simpledesktopcheck) {
    if (simpleCheck()) {
      print("PASSED\n");
      exit 0;
    } else {
      print(">>FAILED<<\n");
      exit 1;
    }
  } elsif ($printhelp) {
    print("usage/help:\n");
    print("--printmenu            prints out the kde-menu structure (will take a while)\n");
    print("--desktopfiles         prints the .desktop-files with absolute path\n");
    print("--simplecheck          does a (quick) simple check of visible .desktop-files\n");
    print("--extendedcheck        does a more extended check of the visible .desktop-files\n");
    print("--completecheck        does an extended check on all found .desktop-files\n");
    print("--walkthrough          walks through all menu (be carefull with this!)\n");
    print("--help                 this help message\n");
  } else {
    print STDERR "ERROR: unknown option(s) ... use --help\n";
    exit 1;
  }
} else {
  print STDERR "ERROR: could not parse commandline-args\n";
  exit 1;
}

#printMenustructure();
#printAppids();
#printDesktopfiles();
#printExecs();
__END__

=head1 NAME

kdeDesktopCheck - short description of your program

=head1 SYNOPSIS

 how to use your program

=head1 DESCRIPTION

 long description of your program

=head1 SEE ALSO

 need to know things before somebody uses your program

=head1 AUTHOR

 Frank Seidel

=cut
